using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using SportReminder.Properties;
using System.Drawing;
using System.Threading;

namespace SportReminder
{
	class MessageFormManager
	{
		MessageForm[] forms;
		NotifyIcon notifyIcon = new NotifyIcon();
		ContextMenuStrip notifyIconMenuStrip = new ContextMenuStrip();
		Settings ApplicationSettings = new Settings();
		System.Windows.Forms.Timer movingTimer;
		System.Windows.Forms.Timer comebackTimer;
		Thread thisThread;

		/// <summary>
		/// The current MessageForm BackColor, if it is applied.
		/// </summary>
		Color currentColor;
		/// <summary>
		/// Counter variable for skipped color frames. One color frame is one step down by the leading MessageForm.
		/// </summary>
		int skippedColorFrames = 0;
		/// <summary>
		/// Indicates the number of step how many color frames have to be skipped before recalculating and switchen currentColor. This field is constant.
		/// </summary>
		const int colorFrameSkip = 4;
		/// <summary>
		/// Indicates how many movingTimer Elapsed event raises shall be ignored before processing one. This field is constant.
		/// </summary>
		const int movTimerStepSkip = 10;



		public MessageFormManager()
		{
			thisThread = Thread.CurrentThread;
			notifyIcon.Text = "Sport Reminder";
			notifyIcon.Icon = SportReminder.Properties.Resources.ApplicationIcon;
			notifyIcon.Visible = true;
			notifyIcon.ContextMenuStrip = notifyIconMenuStrip;
			notifyIconMenuStrip.Items.Add("Done").Click += new EventHandler(doneMenuItem_Click);
			notifyIconMenuStrip.Items.Add("Settings").Click += new EventHandler(settingsMenuItem_Click);
			notifyIconMenuStrip.Items.Add("Exit").Click += new EventHandler(exitMenuItem_Click);
			notifyIconMenuStrip.Opening += new System.ComponentModel.CancelEventHandler(notifyIconMenuStrip_Opening);
			notifyIconMenuStrip.Closed += new ToolStripDropDownClosedEventHandler(notifyIconMenuStrip_Closed);

			ApplicationSettings.Reload();
			ApplicationSettings.SettingChanging += new System.Configuration.SettingChangingEventHandler(ApplicationSettings_SettingChanging);

			movingTimer = new System.Windows.Forms.Timer();
			movingTimer.Interval = 10 * movTimerStepSkip;
			movingTimer.Tick += new EventHandler(movTimer_Elapsed);
			comebackTimer = new System.Windows.Forms.Timer();
			comebackTimer.Tick += new EventHandler(cbTimer_Elapsed);

			Application.ApplicationExit += new EventHandler(Application_ApplicationExit);

			comebackTimer.Interval = (int)ApplicationSettings.ComebackDelay;
			comebackTimer.Start();
		}

		void movTimer_Elapsed(object sender, EventArgs e)
		{
			//if (skippedMovTimerSteps++ == movTimerStepSkip)
			//{
				//skippedMovTimerSteps = 0;
				MoveForms(null);
			//}
		}

		void cbTimer_Elapsed(object sender, EventArgs e)
		{
			ShowForms(null);
		}

		delegate void SetTopMostCallBack(Form form, bool value);

		void SetTopMost(Form form, bool value)
		{
			if (form.InvokeRequired)
			{
				form.Invoke(new SetTopMostCallBack(SetTopMost), new object[] { form, value });
			}
			else
			{
				form.TopMost = value;
			}
		}

		delegate void SetLabelTextCallBack(MessageForm form, string value);

		void SetLabelText(MessageForm form, string text)
		{
			if (form.InvokeRequired)
			{
				form.Invoke(new SetLabelTextCallBack(SetLabelText), new object[] { form, text });
			}
			else
			{
				form.label1.Text = text;
			}
		}

		delegate void SetLabelFontCallBack(MessageForm form, Font value);

		void SetLabelFont(MessageForm form, Font value)
		{
			if (form.InvokeRequired)
			{
				form.Invoke(new SetLabelFontCallBack(SetLabelFont), new object[] { form, value });
			}
			else
			{
				form.label1.Font = value;
			}
		}

		delegate void SetColorCallBack(MessageForm form, Color value);

		void SetLabelForeColor(MessageForm form, Color value)
		{
			if (form.InvokeRequired)
			{
				form.Invoke(new SetColorCallBack(SetLabelForeColor), new object[] { form, value });
			}
			else
			{
				form.label1.ForeColor = value;
			}
		}

		void SetBackColor(MessageForm form, Color value)
		{
			if (form.InvokeRequired)
			{
				form.Invoke(new SetColorCallBack(SetBackColor), new object[] { form, value });
			}
			else
			{
				form.BackColor = value;
			}
		}

		void SetTransparencyKey(MessageForm form, Color value)
		{
			if (form.InvokeRequired)
			{
				form.Invoke(new SetColorCallBack(SetTransparencyKey), new object[] { form, value });
			}
			else
			{
				form.TransparencyKey = value;
			}
		}

		private void RecreateForms()
		{
			if (forms != null)
			{
				foreach (MessageForm form in forms)
				{
					form.Dispose();
				}
			}
			forms = new MessageForm[Screen.AllScreens.Length];
			for (int i = 0; i < forms.Length; i++)
			{
				forms[i] = new MessageForm();
				forms[i].Left = Screen.AllScreens[i].Bounds.Left + 50;
				if (ApplicationSettings.BackgroundTransparent)
				{
					forms[i].BackColor = Color.Black;
					forms[i].TransparencyKey = Color.Black;
				}
			}
			forms[0].Move += new EventHandler(Form1_Move);
		}

		void Application_ApplicationExit(object sender, EventArgs e)
		{
			notifyIcon.Visible = false;
			notifyIcon.Dispose();
			foreach (MessageForm f in forms)
			{
				f.Close();
				f.Dispose();
			}
			movingTimer.Dispose();
			comebackTimer.Dispose();
			notifyIconMenuStrip.Dispose();
		}

		void notifyIconMenuStrip_Opening(object sender, System.ComponentModel.CancelEventArgs e)
		{
			StopForms();
		}

		void notifyIconMenuStrip_Closed(object sender, ToolStripDropDownClosedEventArgs e)
		{
			if (forms != null && forms.Length > 0 && forms[0].Visible)
			{
				BeginMovingForms();
			}
		}

		void Form1_Move(object sender, EventArgs e)
		{
			if (skippedColorFrames++ == colorFrameSkip)
			{
				skippedColorFrames = 0;
				currentColor = Color.FromArgb(
					decimal.ToByte((decimal)(0x7F + Math.Sin(forms[0].Top / (10 * Math.PI)) * 0x7F)),
					decimal.ToByte((decimal)(0x7F + Math.Sin(forms[0].Top / (15 * Math.PI)) * 0x7F)),
					decimal.ToByte((decimal)(0x7F + Math.Sin(forms[0].Top / (25 * Math.PI)) * 0x7F)));	// cycle color
				for (int i = 0; i < forms.Length; i++)
				{
					if (!ApplicationSettings.BackgroundTransparent)
					{
						forms[i].BackColor = currentColor;
					}
					forms[i].label1.ForeColor = Color.FromArgb(~currentColor.ToArgb());	// complementary color
				}
			}
		}

		void ApplicationSettings_SettingChanging(object sender, System.Configuration.SettingChangingEventArgs e)
		{
			switch (e.SettingName)
			{
				case "BackgroundTransparent":
					if ((bool)e.NewValue == true)
					{
						if (forms != null)
						{
							foreach (MessageForm f in forms)
							{
								f.BackColor = Color.Black;
								f.TransparencyKey = Color.Black;
							}
						}
					}
					else
					{
						if (forms != null)
						{
							foreach (MessageForm f in forms)
							{
								f.TransparencyKey = Color.Transparent;
							}
						}
					}
					break;
				default:
					break;
			}
		}

		delegate void HideFormCallBack(MessageForm form);

		void doneMenuItem_Click(object sender, EventArgs e)
		{
			if (forms != null)
			{
				foreach (MessageForm f in forms)
				{
					f.Hide();
				}
			}
			comebackTimer.Interval = (int)ApplicationSettings.ComebackDelay;
			comebackTimer.Start();
			StopForms();
		}

		delegate void MessageFormMethod(MessageForm form);

		void settingsMenuItem_Click(object sender, EventArgs e)
		{
			StopForms();
			if (forms != null)
			{
				foreach (MessageForm f in forms)
				{
					f.TopMost = false;
				}
			}
			new SettingsForm(ApplicationSettings).ShowDialog();
			if (forms != null)
			{
				foreach (MessageForm f in forms)
				{
					f.TopMost = true;
					f.label1.Text = ApplicationSettings.Message;
					f.label1.Font = ApplicationSettings.MessageFont;
					f.InvalidateFontSize();
				}
			}
			BeginMovingForms();
		}

		void exitMenuItem_Click(object sender, EventArgs e)
		{
			Application.Exit();
		}

		delegate void MethodWithoutParams();

		void MoveForms(object state)
		{
			if (forms != null)
			{
				for (int i = 0; i < forms.Length; i++)
				{
					forms[i].Invoke(new MethodWithoutParams(forms[i].IncreaseLocationTop));
				}
			}
		}

		delegate void MacroCallBack();

		void ShowForms(object state)
		{
			comebackTimer.Stop();
			if (forms == null || Screen.AllScreens.Length != forms.Length)
			{
				RecreateForms();
			}
			foreach (MessageForm f in forms)
			{
				if (f.InvokeRequired)
				{
					f.Invoke(new MethodWithoutParams(f.Show));
				}
				else
				{
					f.Show();
				}
			}
			BeginMovingForms();
		}

		void StopForms()
		{
			movingTimer.Stop();
		}

		void BeginMovingForms()
		{
			movingTimer.Start();
		}
	}
}
