using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace SportReminder
{
	internal partial class SettingsForm : Form
	{
		internal SettingsForm(SportReminder.Properties.Settings Settings)
		{
			InitializeComponent();
			ApplicationSettings = Settings;
		}

		/// <summary>
		/// Stores application settings.
		/// </summary>
		SportReminder.Properties.Settings ApplicationSettings;

		private void minutesNumeric_ValueChanged(object sender, EventArgs e)
		{
			switch (decimal.ToInt64(minutesNumeric.Value))
			{
				case 60:
					minutesNumeric.Value = 0;
					hoursNumeric.Value++;
					break;
				case -1:
					minutesNumeric.Value = minutesNumeric.Maximum - 1;
					hoursNumeric.Value--;
					break;
				default:
					break;
			}
		}

		private void secondsNumeric_ValueChanged(object sender, EventArgs e)
		{
			switch (decimal.ToInt64(secondsNumeric.Value))
			{
				case 60:
					secondsNumeric.Value = 0;
					minutesNumeric.Value++;
					break;
				case -1:
					secondsNumeric.Value = minutesNumeric.Maximum - 1;
					minutesNumeric.Value--;
					break;
				default:
					break;
			}
		}

		private void millisecondsNumeric_ValueChanged(object sender, EventArgs e)
		{
			switch (decimal.ToInt64(millisecondsNumeric.Value))
			{
				case 1000:
					millisecondsNumeric.Value = 0;
					secondsNumeric.Value++;
					break;
				case -1:
					millisecondsNumeric.Value = millisecondsNumeric.Maximum - 1;
					secondsNumeric.Value--;
					break;
				default:
					break;
			}
		}

		private void SettingsForm_Load(object sender, EventArgs e)
		{
			messageTextBox.Text = ApplicationSettings.Message;
			fontDialog1.Font = ApplicationSettings.MessageFont;

			hoursNumeric.Value = decimal.Floor(ApplicationSettings.ComebackDelay / 3600000);
			minutesNumeric.Value = decimal.Floor((ApplicationSettings.ComebackDelay - hoursNumeric.Value * 3600000) / 60000);
			secondsNumeric.Value = decimal.Floor((ApplicationSettings.ComebackDelay - hoursNumeric.Value * 3600000 - minutesNumeric.Value * 60000) / 1000);
			millisecondsNumeric.Value = ApplicationSettings.ComebackDelay - hoursNumeric.Value * 3600000 - minutesNumeric.Value * 60000 - secondsNumeric.Value * 1000;
		}

		private void SettingsForm_FormClosed(object sender, FormClosedEventArgs e)
		{
			if (DialogResult == DialogResult.OK)
			{
				ApplicationSettings.ComebackDelay = decimal.ToInt32(millisecondsNumeric.Value + secondsNumeric.Value * 1000 + minutesNumeric.Value * 60000 + hoursNumeric.Value * 3600000);
				ApplicationSettings.Message = messageTextBox.Text;
				ApplicationSettings.MessageFont = fontDialog1.Font;
				ApplicationSettings.BackgroundTransparent = TransparentBgCheckBox.Checked;
				ApplicationSettings.Save();
			}
		}

		private void FontBtn_Click(object sender, EventArgs e)
		{
			fontDialog1.ShowDialog();
		}
	}
}